from geox.connection import connect_to_endpoint
from geox.entity.api_endpoint import APIEndpoint
from geox.entity.dataset_type import DatasetType
from geox.http_response.http_dataset import HttpReponseDataset


def get_endpoint(dataset_type: str) -> str:
    '''get endpoint using dataset type'''
    if dataset_type == DatasetType.COLLAR:
        return APIEndpoint.GET_COLLAR_DATA
    
    elif dataset_type == DatasetType.SURVEY:
        return APIEndpoint.GET_SURVEY_DATA
    
    elif dataset_type == DatasetType.ALTERATION:
        return APIEndpoint.GET_ALTERATION_DATA
    
    elif dataset_type == DatasetType.ASSAY:
        return APIEndpoint.GET_ASSAY_DATA
    
    elif dataset_type == DatasetType.LITHO:
        return APIEndpoint.GET_LITHO_DATA
    
    elif dataset_type == DatasetType.MINERALISATION:
        return APIEndpoint.GET_MINERALISATION_DATA
    

def get_dataset_data(api_key: str, hash: str, next_index: int, dataset_type: str) -> HttpReponseDataset:
    response = connect_to_endpoint(
        url=get_endpoint(dataset_type),
        method='GET',
        headers={
            "api-key": api_key,
        },
        params={
            'hash': hash,
            'next_index': next_index,
        }
    )
    
    return HttpReponseDataset(response, dataset_type)
    
    


from geox.factory.project_factory import ProjectFactory
from geox.project import Project
from typing import List


class HttpReponseProjects():
    def __init__(self, json_response):
        self.projects: List[Project] = None
        
        self._factory = ProjectFactory()
        self._parser(json_response)
    
    
    def _parser(self, json_response):
        if 'projects' in json_response:
            self.projects = []
            for project_data in json_response['projects']:
                project_obj = self._factory.create(project_data)
                if project_obj: self.projects.append(project_obj)



from geox.connection import connect_to_endpoint
from geox.entity.api_endpoint import APIEndpoint
from geox.http_response.http_initial_auth import HttpReponseInitialAuth


def get_initial_auth(api_key: str) -> HttpReponseInitialAuth:
    response = connect_to_endpoint(
        url=APIEndpoint.GET_INITIAL_AUTH,
        method='GET',
        headers={
            "api-key": api_key,
        }
    )
    
    return HttpReponseInitialAuth(response)


from datetime import datetime
from geox.api_caller.get_initial_auth import get_initial_auth
from geox.api_caller.get_project import get_project
from geox.api_caller.get_projects import get_projects
from geox.exceptions import APIKeyException
from geox.project import Project
from geox.version import VERSION
from typing import List


class GeoX:
    """GeoX main class, initialize using api key"""
    def __init__(self, api_key: str):
        self._check_api_key(api_key)
        
        self.api_key: str = api_key
        
        self.email: str = None
        self.projects: List[Project] = None
        self.num_of_projects: int = None
        
        # private variables
        self._timestamp = datetime.now()
        
        self._initial_auth()
    
    
    def _check_api_key(self, api_key):
        '''checking api key variable'''
        if not api_key: raise APIKeyException('API Key cannot be None')
        if not isinstance(api_key, str): raise APIKeyException('API Key should be string')
    
        
    def _initial_auth(self):
        '''checking api key by initial auth'''
        http_response = get_initial_auth(self.api_key)
        
        # updating email and num of projects
        self.email = http_response.email
        self.num_of_projects = http_response.num_of_projects
        
        
    def read_all_projects(self) -> List[Project]:
        """Read all projects of authenticated user

        Returns:
            List[Project]: List of this user Project
        """
        if not self.email: return
        http_response = get_projects(self.api_key)
        self.projects = http_response.projects
        for project in self.projects:
            project.set_api_key(self.api_key)
            
        return self.projects


    def read_project(self, project_id: str) -> Project:
        """Read specific project

        Args:
            project_id (str): project id

        Returns:
            Project: Project object
        """
        if not self.email: return
        http_response = get_project(self.api_key, project_id)
        if http_response.project: http_response.project.set_api_key(self.api_key)
        return http_response.project


    def version(self) -> str:
        """Read this library version

        Returns:
            str: version
        """
        return VERSION




from datetime import datetime


def convert_iso_8601_to_datetime(datetime_str: str) -> datetime:
    '''time converter'''
    try:
        datetime_obj = datetime.strptime(datetime_str, "%Y-%m-%dT%H:%M:%S.%f")
        
    except ValueError as e:
        datetime_obj = None
        
    return datetime_obj